@@ -4,6 +4,7 @@
 
 open OUnit
 open Zeroinstall.General
+open Support
 open Support.Common
 module Config = Zeroinstall.Config
 module U = Support.Utils
@@ -40,7 +41,7 @@ let pp_path f xs =
 let load_first_exn system rel_path search_path =
   match Support.Basedir.load_first system rel_path search_path with
   | Some p -> p
-  | None -> raise_safe "Resource %S not found in %a" rel_path pp_path search_path
+  | None -> Safe_exn.failf "Resource %S not found in %a" rel_path pp_path search_path
 
 module RealSystem = Support.System.RealSystem(Unix)
 let real_system = new RealSystem.real_system
@@ -48,7 +49,10 @@ let real_system = new RealSystem.real_system
 let real_spawn_handler ?env args cin cout cerr =
   real_system#create_process ?env args cin cout cerr
 
-let build_dir = Filename.dirname @@ Filename.dirname Sys.argv.(0)
+let tests_dir = Utils.abspath real_system "."
+let build_dir = Filename.dirname tests_dir
+let test_data x = tests_dir +/ "data" +/ x
+let test_0install = tests_dir +/ "0install"
 
 let make_stat st_perm kind =
   let open Unix in {
@@ -67,8 +71,8 @@ let make_stat st_perm kind =
   }
 
 let flush_all () =
-  Pervasives.flush Pervasives.stdout;
-  Pervasives.flush Pervasives.stderr
+  flush stdout;
+  flush stderr
 
 let capture_stdout ?(include_stderr=false) fn =
   let open Unix in
@@ -137,14 +141,23 @@ let () =
     | _ -> None
   )
 
-let ocaml_dir = Sys.getcwd ()
-let src_dir = Filename.dirname ocaml_dir
-let tests_dir = ocaml_dir +/ "tests"
-let test_0install = U.realpath real_system (build_dir +/ "0install")
+let collect_output fn =
+  let b = Buffer.create 100 in
+  let f = Format.formatter_of_buffer b in
+  fn f;
+  Buffer.contents b
+
+let check_no_output fn =
+  let b = Buffer.create 100 in
+  let f = Format.formatter_of_buffer b in
+  let r = fn f in
+  let out = Buffer.contents b in
+  if out = "" then r
+  else assert_failure (Printf.sprintf "Expected no output, but got %S" out)
 
 class fake_system ?(portable_base=true) tmpdir =
-  let extra_files : dentry StringMap.t ref = ref StringMap.empty in
-  let hidden_files = ref StringSet.empty in
+  let extra_files : dentry XString.Map.t ref = ref XString.Map.empty in
+  let hidden_files = ref XString.Set.empty in
   let redirect_writes = ref None in
 
   let read_ok = ref [] in   (* Always allow reading from these directories *)
@@ -156,60 +169,42 @@ class fake_system ?(portable_base=true) tmpdir =
     if Filename.is_relative path then path
     else (
       try
-        match StringMap.find_nf path !extra_files with
+        match XString.Map.find path !extra_files with
         | Dir _ -> path
         | File (_mode, redirect_path) -> redirect_path
       with Not_found ->
-        if not (U.starts_with path "/home") then path
-        else if U.starts_with path src_dir then path
+        if not (XString.starts_with path "/home") then path
+        else if XString.starts_with path tests_dir then path
         else (
           if !read_ok |> List.exists (fun dir ->
-            U.starts_with path dir || U.starts_with dir path) then path
-          else raise_safe "Attempt to read from '%s'" path
+            XString.starts_with path dir || XString.starts_with dir path) then path
+          else Safe_exn.failf "Attempt to read from '%s'" path
         )
     ) in
 
   let check_write path =
     match !redirect_writes with
-    | Some (from, target) when U.starts_with path from ->
-        target +/ (U.string_tail path (String.length from))
+    | Some (from, target) when XString.starts_with path from ->
+        target +/ (XString.tail path (String.length from))
     | _ ->
         match tmpdir with
-        | Some dir when U.starts_with path dir -> path
-        | Some dir -> raise_safe "Attempt to write to %s (not in %s)" path dir
-        | None -> raise_safe "Attempt to write to '%s' (no tmpdir)" path in
+        | Some dir when XString.starts_with path dir -> path
+        | Some dir -> Safe_exn.failf "Attempt to write to %s (not in %s)" path dir
+        | None -> Safe_exn.failf "Attempt to write to '%s' (no tmpdir)" path in
 
   (* It's OK to check whether these paths exists. We just say they don't,
      unless they're in extra_files (check there first). *)
   let hidden_subtree path =
-    U.starts_with path "/var/lib" ||
-    U.starts_with path "/var/log" in
+    XString.starts_with path "/var/lib" ||
+    XString.starts_with path "/var/log" in
 
   object (self : #system)
     val now = ref @@ 101. *. days
-    val mutable env = StringMap.empty
-    val mutable stdout = None
+    val mutable env = XString.Map.empty
     val mutable spawn_handler = None
     val mutable allow_spawn_detach = false
     val mutable device_boundary = None    (* Reject renames across here to simulate a mount *)
 
-    method collect_output (fn : unit -> unit) =
-      let old_stdout = stdout in
-      let b = Buffer.create 100 in
-      stdout <- Some b;
-      U.finally_do (fun () -> stdout <- old_stdout) () fn;
-      Buffer.contents b
-
-    method print_string s =
-      match stdout with
-      | None -> real_system#print_string s
-      | Some b -> Buffer.add_string b s
-
-    method std_formatter =
-      match stdout with
-      | None -> real_system#std_formatter
-      | Some b -> Format.formatter_of_buffer b
-
     val mutable argv = [| test_0install |]
 
     method argv = argv
@@ -227,14 +222,14 @@ class fake_system ?(portable_base=true) tmpdir =
     method mkdir path mode =
       let path = check_write path in
       let parent = Filename.dirname path in
-      if not (real_system#file_exists parent) && StringMap.mem parent !extra_files then
+      if not (real_system#file_exists parent) && XString.Map.mem parent !extra_files then
         U.makedirs real_system parent 0o700;
       real_system#mkdir path mode
 
     method rename source target =
       device_boundary |> if_some (fun device_boundary ->
-        let a_dev = U.starts_with source device_boundary in
-        let b_dev = U.starts_with target device_boundary in
+        let a_dev = XString.starts_with source device_boundary in
+        let b_dev = XString.starts_with target device_boundary in
         if a_dev <> b_dev then
           raise (Unix.Unix_error (Unix.EXDEV, "rename", target))
       );
@@ -243,16 +238,15 @@ class fake_system ?(portable_base=true) tmpdir =
     method set_device_boundary b = device_boundary <- b
 
     method readdir path =
-      try
-        match StringMap.find_nf path !extra_files with
-        | Dir (_mode, items) -> Success (Array.of_list items)
-        | _ -> failwith "Not a directory"
-      with Not_found -> real_system#readdir (check_read path)
+      match XString.Map.find_opt path !extra_files with
+      | Some (Dir (_mode, items)) -> Ok (Array.of_list items)
+      | Some _ -> failwith "Not a directory"
+      | None -> real_system#readdir (check_read path)
 
     method symlink ~target ~newlink = real_system#symlink ~target ~newlink:(check_write newlink)
 
     method readlink path =
-      if StringMap.mem path !extra_files then None    (* Not a link *)
+      if XString.Map.mem path !extra_files then None    (* Not a link *)
       else real_system#readlink (check_read path)
 
     method chmod path mode = real_system#chmod (check_write path) mode
@@ -260,34 +254,32 @@ class fake_system ?(portable_base=true) tmpdir =
     method file_exists path =
       if path = "/usr/bin/0install" then true
       else if path = "C:\\Windows\\system32\\0install.exe" then true
-      else if StringMap.mem path !extra_files then true
-      else if StringSet.mem path !hidden_files then (log_info "hide %s" path; false)
+      else if XString.Map.mem path !extra_files then true
+      else if XString.Set.mem path !hidden_files then (log_info "hide %s" path; false)
       else if tmpdir = None then false
       else if hidden_subtree path then false
       else real_system#file_exists (check_read path)
 
     method lstat path =
-      if StringSet.mem path !hidden_files then None
+      if XString.Set.mem path !hidden_files then None
       else (
-        try
-          let open Unix in
-          match StringMap.find_nf path !extra_files with
-          | Dir (mode, _items) -> Some (make_stat mode S_DIR)
-          | File (_mode, target) -> real_system#lstat target
-        with Not_found ->
+        let open Unix in
+        match XString.Map.find_opt path !extra_files with
+        | Some Dir (mode, _items) -> Some (make_stat mode S_DIR)
+        | Some File (_mode, target) -> real_system#lstat target
+        | None ->
           if hidden_subtree path then None
           else real_system#lstat (check_read path)
       )
 
     method stat path =
-      if StringSet.mem path !hidden_files then None
+      if XString.Set.mem path !hidden_files then None
       else (
-        try
-          let open Unix in
-          match StringMap.find_nf path !extra_files with
-          | Dir (mode, _items) -> Some (make_stat mode S_DIR)
-          | File (_mode, target) -> real_system#stat target
-        with Not_found ->
+        let open Unix in
+        match XString.Map.find_opt path !extra_files with
+        | Some Dir (mode, _items) -> Some (make_stat mode S_DIR)
+        | Some File (_mode, target) -> real_system#stat target
+        | None ->
           if tmpdir = None then None
           else if hidden_subtree path then None
           else real_system#stat (check_read path)
@@ -312,9 +304,10 @@ class fake_system ?(portable_base=true) tmpdir =
       if allow_spawn_detach then (
         ignore search_path;
         (* For testing, we run in-process to allow tests interceptors to work, etc. *)
-        if List.hd argv <> test_0install then raise_safe "spawn_detach with %s (not %s)" (List.hd argv) test_0install;
+        if List.hd argv <> test_0install then Safe_exn.failf "spawn_detach with %s (not %s)" (List.hd argv) test_0install;
         let config = Zeroinstall.Config.get_default_config (self :> system) test_0install in
-        Cli.handle config (List.tl argv)
+        let devnull = Format.formatter_of_buffer (Buffer.create 100) in
+        Cli.handle ~stdout:devnull config (List.tl argv)
       ) else raise (Would_spawn (search_path, env, argv))
 
     method create_process ?env args new_stdin new_stdout new_stderr =
@@ -338,15 +331,15 @@ class fake_system ?(portable_base=true) tmpdir =
 
     method environment =
       let to_str (name, value) = name ^ "=" ^ value in
-      Array.of_list (List.map to_str @@ StringMap.bindings env)
+      Array.of_list (List.map to_str @@ XString.Map.bindings env)
 
-    method getenv name = StringMap.find name env
+    method getenv name = XString.Map.find_opt name env
 
     method putenv name value =
-      env <- StringMap.add name value env
+      env <- XString.Map.add name value env
 
     method unsetenv name =
-      env <- StringMap.remove name env
+      env <- XString.Map.remove name env
 
     method platform =
       let open Platform in {
@@ -356,18 +349,16 @@ class fake_system ?(portable_base=true) tmpdir =
       }
 
     method add_file path redirect_target =
-      extra_files := StringMap.add path (File (0o644, redirect_target)) !extra_files;
+      extra_files := XString.Map.add path (File (0o644, redirect_target)) !extra_files;
       let rec add_parent path =
         let parent = Filename.dirname path in
         if (parent <> path) then (
           let leaf = Filename.basename path in
           let () =
-            try
-              match StringMap.find_nf parent !extra_files with
-              | Dir (mode, items) -> extra_files := StringMap.add parent (Dir (mode, leaf :: items)) !extra_files
-              | _ -> failwith parent
-            with Not_found ->
-              self#add_dir parent [leaf] in
+            match XString.Map.find_opt parent !extra_files with
+            | Some Dir (mode, items) -> extra_files := XString.Map.add parent (Dir (mode, leaf :: items)) !extra_files
+            | Some _ -> failwith parent
+            | None -> self#add_dir parent [leaf] in
           add_parent parent
         ) in
       add_parent path
@@ -377,15 +368,15 @@ class fake_system ?(portable_base=true) tmpdir =
       redirect_writes := Some (src ^ Filename.dir_sep, target ^ Filename.dir_sep)
 
     method add_dir path items =
-      extra_files := StringMap.add path (Dir (0o755, items)) !extra_files;
+      extra_files := XString.Map.add path (Dir (0o755, items)) !extra_files;
       let add_file leaf =
         let full = path +/ leaf in
-        if not (StringMap.mem full !extra_files) then
+        if not (XString.Map.mem full !extra_files) then
           self#add_file full "" in
       List.iter add_file items
 
     method hide_path path =
-      hidden_files := StringSet.add path !hidden_files
+      hidden_files := XString.Set.add path !hidden_files
 
     method running_as_root = false
 
@@ -431,8 +422,8 @@ class null_ui =
   object (_ : #Zeroinstall.Ui.ui_handler as 'a)
     constraint 'a = #Zeroinstall.Progress.watcher
     inherit Zeroinstall.Console.batch_ui
-    method! confirm_keys feed_url _xml = raise_safe "confirm_keys: %s" (Zeroinstall.Feed_url.format_url feed_url)
-    method! confirm msg = raise_safe "confirm: %s" msg
+    method! confirm_keys feed_url _xml = Safe_exn.failf "confirm_keys: %s" (Zeroinstall.Feed_url.format_url feed_url)
+    method! confirm msg = Safe_exn.failf "confirm: %s" msg
   end
 
 let null_ui = new null_ui
@@ -489,7 +480,7 @@ let fake_log =
     method assert_contains expected =
       let re = Str.regexp expected in
       if not (List.exists (fun (_ex, _lvl, msg) -> Str.string_match re msg 0) record) then
-        raise_safe "Expected log message matching '%s'" expected
+        Safe_exn.failf "Expected log message matching '%s'" expected
 
     method record x = record <- x :: record
   end
@@ -516,34 +507,29 @@ let equal_str_lists = assert_equal ~printer:format_list
 let assert_str_equal = assert_equal ~printer:(fun x -> x)
 
 let assert_raises_safe expected_msg (fn:unit Lazy.t) =
-  try Lazy.force fn; assert_failure ("Expected Safe_exception " ^ expected_msg)
-  with Safe_exception (msg, _) ->
+  try Lazy.force fn; assert_failure ("Expected Safe_exn.T " ^ expected_msg)
+  with Safe_exn.T e ->
+    let msg = Safe_exn.msg e in
     if not (Str.string_match (Str.regexp expected_msg) msg 0) then
-      raise_safe "Error '%s' does not match regexp '%s'" msg expected_msg
+      Safe_exn.failf "Error '%s' does not match regexp '%s'" msg expected_msg
 
 let assert_raises_safe_lwt expected_msg fn =
   Lwt.catch
     (fun () ->
        fn () >|= fun () ->
-       assert_failure ("Expected Safe_exception " ^ expected_msg)
+       assert_failure ("Expected Safe_exn.T " ^ expected_msg)
     )
     (function
-      | Safe_exception (msg, _) ->
+      | Safe_exn.T e ->
+        let msg = Safe_exn.msg e in
         if Str.string_match (Str.regexp expected_msg) msg 0 then
           Lwt.return ()
         else
-          raise_safe "Error '%s' does not match regexp '%s'" msg expected_msg
+          Safe_exn.failf "Error '%s' does not match regexp '%s'" msg expected_msg
       | ex -> Lwt.fail ex
     )
 
-let temp_dir_name =
-  (* Filename.get_temp_dir_name doesn't exist under 3.12 *)
-  U.realpath real_system
-    begin try Sys.getenv "TEMP" with Not_found ->
-      match Sys.os_type with
-        | "Unix" | "Cygwin" -> "/tmp"
-        | "Win32" -> "."
-        | _ -> failwith "temp_dir_name: unknown filesystem" end
+let temp_dir_name = Filename.get_temp_dir_name ()
 
 let with_tmpdir fn () =
   U.finally_do reset_env ()
@@ -577,24 +563,24 @@ let get_fake_config ?portable_base tmpdir =
         dir in
   system#putenv "HOME" home;
   if on_windows then (
-    system#add_file (src_dir +/ "0install-runenv.exe") (build_dir +/ "0install-runenv.exe");
+    system#add_file (build_dir +/ "0install-runenv.exe") (tests_dir +/ "0install-runenv.exe");
     system#putenv "PATH" ("c:\\bin;" ^ Sys.getenv "PATH");
     system#add_dir "c:\\bin" [];
-    system#add_file "c:\\bin\\0install-runenv.exe" (build_dir +/ "0install-runenv.exe");
-    system#add_file "c:\\bin\\0install.exe" (build_dir +/ "0install.exe");
-    system#add_file "c:\\bin\\0launch.exe" (build_dir +/ "0install.exe");
-    system#add_file "c:\\bin\\0alias.exe" (build_dir +/ "0install.exe");
+    system#add_file "c:\\bin\\0install-runenv.exe" (tests_dir +/ "0install-runenv.exe");
+    system#add_file "c:\\bin\\0install.exe" (tests_dir +/ "0install.exe");
+    system#add_file "c:\\bin\\0launch.exe" (tests_dir +/ "0install.exe");
+    system#add_file "c:\\bin\\0alias.exe" (tests_dir +/ "0install.exe");
     match tmpdir with
     | Some tmpdir -> Support.Windows_api.windowsAPI := Some (fake_win_dirs tmpdir);
     | None -> ()
   ) else (
     system#putenv "PATH" @@ (home +/ "bin") ^ ":" ^ (Sys.getenv "PATH");
-    system#add_file test_0install (build_dir +/ "0install");
-    system#add_file "/usr/bin/0install" (build_dir +/ "0install");
-    system#add_file "/usr/bin/0launch" (build_dir +/ "0install");
-    system#add_file "/usr/bin/0alias" (build_dir +/ "0install");
+    system#add_file test_0install (tests_dir +/ "0install");
+    system#add_file "/usr/bin/0install" (tests_dir +/ "0install");
+    system#add_file "/usr/bin/0launch" (tests_dir +/ "0install");
+    system#add_file "/usr/bin/0alias" (tests_dir +/ "0install");
   );
-  system#allow_read src_dir;
+  system#allow_read tests_dir;
   (* Allow reading from all PATH directories *)
   Str.split_delim U.re_path_sep (system#getenv "PATH" |? lazy (failwith "PATH")) |> List.iter (fun dir ->
     system#allow_read dir
@@ -619,7 +605,8 @@ let assert_error_contains expected (fn:unit -> unit) =
   try
     fn ();
     assert_failure (Printf.sprintf "Expected error '%s' but got success!" expected)
-  with Safe_exception (msg, _) ->
+  with Safe_exn.T e ->
+    let msg = Safe_exn.msg e in
     assert_contains expected msg
 
 (** Read all input from a channel. *)
