@@ -6,6 +6,7 @@
 
 open Zeroinstall.General
 open Zeroinstall
+open Support
 open Support.Common
 open OUnit
 module Impl = Zeroinstall.Impl
@@ -25,7 +26,6 @@ let assert_error_contains expected fn =
   )
 
 let test_0install = Fake_system.test_0install
-let feed_dir = U.abspath Fake_system.real_system (".." +/ "tests")
 
 exception Ok
 
@@ -37,12 +37,12 @@ let handle_download_impls config pending_digests impls =
           let digest_str =
             digests |> U.first_match (fun digest ->
               let digest_str = Zeroinstall.Manifest.format_digest digest in
-              if StringSet.mem digest_str !pending_digests then Some digest_str else None
+              if XString.Set.mem digest_str !pending_digests then Some digest_str else None
             ) in
           match digest_str with
-          | None -> raise_safe "Digest not expected!"
+          | None -> Safe_exn.failf "Digest not expected!"
           | Some digest_str ->
-              pending_digests := StringSet.remove digest_str !pending_digests;
+              pending_digests := XString.Set.remove digest_str !pending_digests;
               let user_store = List.hd config.stores in
               U.makedirs config.system (user_store +/ digest_str) 0o755;
               log_info "Added %s to stores" digest_str
@@ -53,8 +53,8 @@ let handle_download_impls config pending_digests impls =
 let impl_from_json config = (function
   | `Assoc [("id", `String id); ("from-feed", `String feed_url)] ->
       let parsed = Zeroinstall.Feed_url.parse_non_distro feed_url in
-      let feed = Zeroinstall.Feed_cache.get_cached_feed config parsed |? lazy (raise_safe "Not cached: %s" feed_url) in
-      StringMap.find_safe id feed.F.implementations
+      let feed = Zeroinstall.Feed_cache.get_cached_feed config parsed |? lazy (Safe_exn.failf "Not cached: %s" feed_url) in
+      XString.Map.find_safe id feed.F.implementations
   | _ -> assert false
 )
 
@@ -68,16 +68,16 @@ let write_script (system:#system) launcher_script interface_uri =
   )
 
 class fake_slave _config =
-  let pending_feed_downloads = ref StringMap.empty in
+  let pending_feed_downloads = ref XString.Map.empty in
 
   let handle_download ?if_slow:_ ?size:_ ?modification_time:_ ch url =
     let contents =
-      if U.starts_with url "https://keylookup.appspot.com/key/" then (
+      if XString.starts_with url "https://keylookup.0install.net/key/" then (
         "<key-lookup><item vote='good'>Looks legit</item></key-lookup>"
       ) else (
-        StringMap.find_safe url !pending_feed_downloads
+        XString.Map.find_safe url !pending_feed_downloads
       ) in
-    pending_feed_downloads := StringMap.remove url !pending_feed_downloads;
+    pending_feed_downloads := XString.Map.remove url !pending_feed_downloads;
     output_string ch contents;
     `Success |> Lwt.return in
 
@@ -86,7 +86,7 @@ class fake_slave _config =
       Zeroinstall.Downloader.interceptor := Some handle_download
 
     method allow_download url contents =
-      pending_feed_downloads := StringMap.add url contents !pending_feed_downloads
+      pending_feed_downloads := XString.Map.add url contents !pending_feed_downloads
   end
 
 let run_0install ?stdin ?(binary=test_0install) ?(include_stderr=false) fake_system ?(exit=0) args =
@@ -94,8 +94,9 @@ let run_0install ?stdin ?(binary=test_0install) ?(include_stderr=false) fake_sys
     Fake_system.fake_log#reset;
     fake_system#set_argv @@ Array.of_list (binary :: args);
     Fake_system.capture_stdout ~include_stderr (fun () ->
-      try Main.main (fake_system : Fake_system.fake_system :> system); assert (exit = 0)
-      with System_exit n -> assert_equal ~msg:"exit code" n exit
+        let stdout = Format.std_formatter in
+        try Main.main ~stdout (fake_system : Fake_system.fake_system :> system); assert (exit = 0)
+        with System_exit n -> assert_equal ~msg:"exit code" n exit
     )
   ) in
   match stdin with
@@ -115,7 +116,7 @@ let check_man fake_system args expected =
       | x -> x in
     Fake_system.equal_str_lists expected man_args
 
-let generic_archive = U.handle_exceptions (U.read_file Fake_system.real_system) @@ feed_dir +/ "HelloWorld.tgz"
+let generic_archive = U.handle_exceptions (U.read_file Fake_system.real_system) @@ Fake_system.test_data "HelloWorld.tgz"
 
 let selections_of_string ?path s =
   `String (0, s) |> Xmlm.make_input |> Q.parse_input path |> Selections.create
@@ -124,19 +125,19 @@ let suite = "0install">::: [
   "select">:: Fake_system.with_tmpdir (fun tmpdir ->
     let (_config, fake_system) = Fake_system.get_fake_config (Some tmpdir) in
     let system = (fake_system :> system) in
-    fake_system#add_file (tmpdir +/ "cache" +/ "interfaces" +/ "http%3a%2f%2fexample.com%3a8000%2fHello.xml") (feed_dir +/ "Hello.xml");
+    fake_system#add_file (tmpdir +/ "cache" +/ "interfaces" +/ "http%3a%2f%2fexample.com%3a8000%2fHello.xml") (Fake_system.test_data "Hello.xml");
     fake_system#add_dir (tmpdir +/ "cache" +/ "implementations") ["sha1=3ce644dc725f1d21cfcf02562c76f375944b266a"];
     fake_system#add_file "/lib/ld-linux.so.2" "/";    (* Enable multi-arch *)
 
     (* In --offline mode we select from the cached feed *)
     fake_system#set_argv [| test_0install; "-o"; "select"; "http://example.com:8000/Hello.xml" |];
-    let output = fake_system#collect_output (fun () -> Main.main system) in
-    assert (U.starts_with output "- URI: http://example.com:8000/Hello.xml");
+    let output = Fake_system.collect_output (fun stdout -> Main.main ~stdout system) in
+    assert (XString.starts_with output "- URI: http://example.com:8000/Hello.xml");
 
     (* In online mode, we spawn a background process because we don't have a last-checked timestamp *)
     fake_system#set_argv [| test_0install; "select"; "http://example.com:8000/Hello.xml" |];
     let () =
-      try failwith @@ fake_system#collect_output (fun () -> Main.main system)
+      try failwith @@ Fake_system.collect_output (fun stdout -> Main.main ~stdout system)
       with Fake_system.Would_spawn (_, _, args) ->
         Fake_system.equal_str_lists
          ["select"; "--refresh"; "--console"; "-v"; "--command"; "run"; "http://example.com:8000/Hello.xml"]
@@ -144,7 +145,7 @@ let suite = "0install">::: [
 
     (* Download succeeds (does nothing, as it's already cached *)
     fake_system#set_argv [| test_0install; "-o"; "download"; "http://example.com:8000/Hello.xml" |];
-    let output = fake_system#collect_output (fun () -> Main.main system) in
+    let output = Fake_system.collect_output (fun stdout -> Main.main ~stdout system) in
     assert_str_equal "" output;
 
     (* Use --console --offline --refresh to force us to use the Python *)
@@ -153,8 +154,9 @@ let suite = "0install">::: [
     fake_system#set_spawn_handler (Some my_spawn_handler);
     fake_system#set_argv [| test_0install; "-cor"; "download"; "http://example.com:8000/Hello.xml"; "--version"; "2" |];
     let () =
-      try Main.main system; assert false
-      with Safe_exception (msg, _) ->
+      try Fake_system.check_no_output (fun stdout -> Main.main ~stdout system); assert false
+      with Safe_exn.T e ->
+        let msg = Safe_exn.msg e in
         assert_str_equal (
           "Can't find all required implementations:\n" ^
           "- http://example.com:8000/Hello.xml -> (problem)\n" ^
@@ -172,28 +174,28 @@ let suite = "0install">::: [
 
     let run ?(exit=0) args =
       fake_system#set_argv @@ Array.of_list (test_0install :: "--console" :: args);
-      fake_system#collect_output (fun () ->
-        try Main.main system; assert (exit = 0)
+      Fake_system.collect_output (fun stdout ->
+        try Main.main ~stdout system; assert (exit = 0)
         with System_exit n -> assert_equal ~msg:"exit code" n exit
       ) in
 
     Unix.putenv "DISPLAY" "";
 
     let out = run ~exit:1 ["update"] in
-    assert (U.starts_with out "Usage:");
+    assert (XString.starts_with out "Usage:");
     assert_contains "--message" out;
 
     (* Updating a local feed with no dependencies *)
     let local_file = tmpdir +/ "Local.xml" in
-    U.copy_file system (feed_dir +/ "Local.xml") local_file 0o644;
+    U.copy_file system (Fake_system.test_data "Local.xml") local_file 0o644;
     let out = run ["update"; local_file] in
     assert_contains "No updates found" out;
 
     let fake_slave = new fake_slave config in
     fake_slave#install;
 
-    let binary_feed = Support.Utils.read_file system (feed_dir +/ "Binary.xml") in
-    let test_key = Support.Utils.read_file system (feed_dir +/ "6FCF121BE2390E0B.gpg") in
+    let binary_feed = Support.Utils.read_file system (Fake_system.test_data "Binary.xml") in
+    let test_key = Support.Utils.read_file system (Fake_system.test_data "6FCF121BE2390E0B.gpg") in
 
     (* Using a remote feed for the first time *)
     fake_slave#allow_download "http://example.com/Binary-1.0.tgz" generic_archive;
@@ -209,14 +211,14 @@ let suite = "0install">::: [
     assert_contains "No updates found" out;
 
     (* New binary release available. *)
-    let binary_feed = Support.Utils.read_file system (feed_dir +/ "Binary2.xml") in
+    let binary_feed = Support.Utils.read_file system (Fake_system.test_data "Binary2.xml") in
     fake_slave#allow_download "http://foo/Binary.xml" binary_feed;
     let out = run ["update"; "http://foo/Binary.xml"] in
     assert_contains "Binary.xml: 1.0 -> 1.1" out;
 
     (* Compiling from source for the first time. *)
-    let source_feed = U.read_file system (feed_dir +/ "Source.xml") in
-    let compiler_feed = U.read_file system (feed_dir +/ "Compiler.xml") in
+    let source_feed = U.read_file system (Fake_system.test_data "Source.xml") in
+    let compiler_feed = U.read_file system (Fake_system.test_data "Compiler.xml") in
     fake_slave#allow_download "http://example.com/Source-1.0.tgz" generic_archive;
     fake_slave#allow_download "http://example.com/Compiler-1.0.tgz" generic_archive;
     fake_slave#allow_download "http://foo/Compiler.xml" compiler_feed;
@@ -227,7 +229,7 @@ let suite = "0install">::: [
     assert_contains "Compiler.xml: new -> 1.0" out;
 
     (* New compiler released. *)
-    let new_compiler_feed = U.read_file system (feed_dir +/ "Compiler2.xml") in
+    let new_compiler_feed = U.read_file system (Fake_system.test_data "Compiler2.xml") in
     fake_slave#allow_download "http://foo/Compiler.xml" new_compiler_feed;
     fake_slave#allow_download "http://foo/Binary.xml" binary_feed;
     fake_slave#allow_download "http://foo/Source.xml" source_feed;
@@ -235,7 +237,7 @@ let suite = "0install">::: [
     assert_contains "Compiler.xml: 1.0 -> 1.1" out;
 
     (* A dependency disappears. *)
-    let new_source_feed = U.read_file system (feed_dir +/ "Source-missing-req.xml") in
+    let new_source_feed = U.read_file system (Fake_system.test_data "Source-missing-req.xml") in
     fake_slave#allow_download "http://foo/Compiler.xml" new_compiler_feed;
     fake_slave#allow_download "http://foo/Binary.xml" binary_feed;
     fake_slave#allow_download "http://foo/Source.xml" new_source_feed;
@@ -252,32 +254,33 @@ let suite = "0install">::: [
     let run = run_0install fake_system in
 
     let out = run ~exit:1 ["download"] in
-    assert (U.starts_with out "Usage:");
+    assert (XString.starts_with out "Usage:");
     assert_contains "--show" out;
 
-    let out = run ["download"; (feed_dir +/ "Local.xml"); "--show"] in
+    let out = run ["download"; (Fake_system.test_data "Local.xml"); "--show"] in
     assert_contains "Version: 0.1" out;
 
-    let local_uri = U.abspath system (feed_dir +/ "Local.xml") in
+    let local_uri = U.abspath system (Fake_system.test_data "Local.xml") in
     let out = run ["download"; local_uri; "--xml"] in
     let sels = selections_of_string out in
     let sel = Selections.(get_selected_ex {iface = local_uri; source = false}) sels in
     assert_str_equal "0.1" @@ Element.version sel;
 
     let () =
-      try ignore @@ run ["download"; "--offline"; (feed_dir +/ "selections.xml")]; assert false
-      with Safe_exception (msg, _) ->
+      try ignore @@ run ["download"; "--offline"; (Fake_system.test_data "selections.xml")]; assert false
+      with Safe_exn.T e ->
+        let msg = Safe_exn.msg e in
         assert_contains "Can't download as in offline mode:\nhttp://example.com:8000/Hello.xml 1" msg in
 
     let fake_slave = new fake_slave config in
     fake_slave#install;
     let digest = "sha1=3ce644dc725f1d21cfcf02562c76f375944b266a" in
     fake_slave#allow_download "http://example.com:8000/HelloWorld.tgz" generic_archive;
-    let out = run ["download"; (feed_dir +/ "Hello.xml"); "--show"] in
+    let out = run ["download"; (Fake_system.test_data "Hello.xml"); "--show"] in
     assert_contains digest out;
     assert_contains "Version: 1\n" out;
 
-    let out = run ["download"; "--offline"; (feed_dir +/ "selections.xml"); "--show"] in
+    let out = run ["download"; "--offline"; (Fake_system.test_data "selections.xml"); "--show"] in
     assert_contains digest out;
     assert_contains "Version: 1\n" out;
   );
@@ -292,11 +295,11 @@ let suite = "0install">::: [
     fake_slave#install;
     fake_slave#allow_download "http://example.com:8000/HelloWorld.tgz" generic_archive;
 
-    let hello = Support.Utils.read_file system (feed_dir +/ "Hello.xml") in
-    let key = Support.Utils.read_file system (feed_dir +/ "6FCF121BE2390E0B.gpg") in
+    let hello = Support.Utils.read_file system (Fake_system.test_data "Hello.xml") in
+    let key = Support.Utils.read_file system (Fake_system.test_data "6FCF121BE2390E0B.gpg") in
     fake_slave#allow_download "http://example.com:8000/Hello.xml" hello;
     fake_slave#allow_download "http://example.com:8000/6FCF121BE2390E0B.gpg" key;
-    let out = run ["download"; (feed_dir +/ "selections.xml"); "--show"] in
+    let out = run ["download"; (Fake_system.test_data "selections.xml"); "--show"] in
     assert_contains digest out;
     assert_contains "Version: 1\n" out;
     Lwt.return ()
@@ -308,7 +311,7 @@ let suite = "0install">::: [
 
     fake_system#unsetenv "DISPLAY";
     try ignore @@ run ["run"; "--gui"; "http://foo/d"]; assert false
-    with Safe_exception ("Can't use GUI because $DISPLAY is not set", _) -> ();
+    with Safe_exn.T e when Safe_exn.msg e = "Can't use GUI because $DISPLAY is not set" -> ();
 
     (* --dry-run must prevent us from using the GUI *)
     fake_system#putenv "DISPLAY" ":foo";
@@ -329,12 +332,12 @@ let suite = "0install">::: [
     fake_system#set_time @@ Fake_system.real_system#time;
 
     let out = run ~exit:1 ["add"; "local-app"] in
-    assert (U.starts_with out "Usage:");
+    assert (XString.starts_with out "Usage:");
 
     let out = run ~exit:1 ["destroy"; "local-app"; "uri"] in
-    assert (U.starts_with out "Usage:");
+    assert (XString.starts_with out "Usage:");
 
-    let local_feed = feed_dir +/ "Local.xml" in
+    let local_feed = Fake_system.test_data "Local.xml" in
 
     assert_error_contains "Invalid application name 'local:app'" (fun () ->
       run ["add"; "local:app"; local_feed]
@@ -368,7 +371,7 @@ let suite = "0install">::: [
     );
 
     let out = run ["man"; "--dry-run"; "local-app"] in
-    assert_contains "tests/test-echo.1" out;
+    assert_contains "data/test-echo.1" out;
 
     fake_system#putenv "COMP_CWORD" "2";
     let out = run ["_complete"; "bash"; "0install"; "select"] in
@@ -430,7 +433,7 @@ let suite = "0install">::: [
     let out = run ["_complete"; "bash"; "0install"; "whatchanged"] in
     assert_contains "local-app" out;
     let out = run ~exit:1 ["whatchanged"; "local-app"; "uri"] in
-    assert (U.starts_with out "Usage:");
+    assert (XString.starts_with out "Usage:");
 
     let out = run ["whatchanged"; "local-app"] in
     assert_contains "No previous history to compare against." out;
@@ -489,12 +492,12 @@ let suite = "0install">::: [
     config.freshness <- None;
 
     let out = run ["add"; "--help"] in
-    assert (U.starts_with out "Usage:");
+    assert (XString.starts_with out "Usage:");
 
     Unix.mkdir (tmpdir +/ "bin") 0o700;
     fake_system#putenv "PATH" ((tmpdir +/ "bin") ^ path_sep ^ U.getenv_ex fake_system "PATH");
 
-    let local_feed = feed_dir +/ "Local.xml" in
+    let local_feed = Fake_system.test_data "Local.xml" in
     let data_home = tmpdir +/ "data" in
     let local_copy = data_home +/ "Local.xml" in
 
@@ -528,7 +531,7 @@ let suite = "0install">::: [
     let sels = A.get_selections_may_update tools app |> Lwt_main.run in
     let printer xs =
       let pp_sep f () = Format.pp_print_string f "; " in
-      Format.asprintf "[%a]" (Format.pp_print_list ~pp_sep Element.fmt) xs
+      Format.asprintf "[%a]" (Format.pp_print_list ~pp_sep Element.pp) xs
     in
     assert_equal ~printer [] @@ Zeroinstall.Driver.get_unavailable_selections config ~distro sels;
     assert_equal 400.0 (A.get_times system app).A.last_solve;
@@ -545,7 +548,7 @@ let suite = "0install">::: [
     fake_system#allow_spawn_detach true;
     (* Local feed is updated; now requires a download *)
     system#unlink (app +/ "last-check-attempt");
-    let hello_feed = (feed_dir +/ "Hello.xml") in
+    let hello_feed = (Fake_system.test_data "Hello.xml") in
     system#set_mtime (app +/ "last-solve") 400.0;
     U.copy_file system hello_feed local_copy 0o600;
     Fake_system.collect_logging (fun () ->
@@ -556,7 +559,7 @@ let suite = "0install">::: [
       Fake_system.fake_log#assert_contains "Still not connected to network. Giving up on background update.";
       match Fake_system.fake_log#pop_warnings with
       | [ w ] -> assert_contains "Error starting background check for updates" w
-      | ws -> raise_safe "Got %d warnings" (List.length ws) in
+      | ws -> Safe_exn.failf "Got %d warnings" (List.length ws) in
 
     (* Selections changed, but no download required *)
     let data = U.read_file system local_copy in
@@ -582,8 +585,8 @@ let suite = "0install">::: [
     let binary_iface = "http://foo/Binary.xml" in
     let run ?(exit=0) args =
       fake_system#set_argv @@ Array.of_list (test_0install :: "--console" :: args);
-      fake_system#collect_output (fun () ->
-        try Main.main config.system; assert (exit = 0)
+      Fake_system.collect_output (fun stdout ->
+        try Main.main ~stdout config.system; assert (exit = 0)
         with System_exit n -> assert_equal ~msg:"exit code" n exit
       ) in
 
@@ -593,7 +596,7 @@ let suite = "0install">::: [
     assert_contains "usage:" @@ String.lowercase_ascii out;
     assert_contains "NEW-FEED" out;
 
-    let out = fake_system#with_stdin "\n" (lazy (run ["add-feed"; (feed_dir +/ "Source.xml")])) in
+    let out = fake_system#with_stdin "\n" (lazy (run ["add-feed"; (Fake_system.test_data "Source.xml")])) in
     assert_contains "Add as feed for 'http://foo/Binary.xml'" out;
     let iface_config = FC.load_iface_config config binary_iface in
     assert_equal 1 @@ List.length iface_config.FC.extra_feeds;
@@ -604,13 +607,13 @@ let suite = "0install">::: [
     assert_contains "file\n" @@ Test_completion.do_complete fake_system "zsh" ["remove-feed"; ""] 2;
     assert_contains "Source.xml" @@ Test_completion.do_complete fake_system "zsh" ["remove-feed"; binary_iface] 3;
 
-    let out = fake_system#with_stdin "\n" (lazy (run ["remove-feed"; (feed_dir +/ "Source.xml")])) in
+    let out = fake_system#with_stdin "\n" (lazy (run ["remove-feed"; (Fake_system.test_data "Source.xml")])) in
     assert_contains "Remove as feed for 'http://foo/Binary.xml'" out;
     let iface_config = FC.load_iface_config config binary_iface in
     assert_equal 0 @@ List.length iface_config.FC.extra_feeds;
 
     let tmp_feed, ch = Filename.open_temp_file "0install-" "-test-feed" in
-    feed_dir +/ "Source.xml" |> fake_system#with_open_in [Open_binary] (fun source_ch ->
+    Fake_system.test_data "Source.xml" |> fake_system#with_open_in [Open_binary] (fun source_ch ->
       U.copy_channel source_ch ch;
     );
     close_out ch;
@@ -635,7 +638,7 @@ let suite = "0install">::: [
   "digest">:: Fake_system.with_fake_config (fun (config, fake_system) ->
     skip_if on_windows "Uses tar";
     let run args = run_0install fake_system args in
-    let hw = feed_dir +/ "HelloWorld.tgz" in
+    let hw = Fake_system.test_data "HelloWorld.tgz" in
 
     let out = run ["digest"; "--algorithm=sha1"; hw] in
     assert_str_equal "sha1=3ce644dc725f1d21cfcf02562c76f375944b266a\n" out;
@@ -663,11 +666,11 @@ let suite = "0install">::: [
     assert_contains "Usage:" out;
     assert_contains "--xml" out;
 
-    let out = run_0install fake_system ["show"; feed_dir +/ "selections.xml"] in
+    let out = run_0install fake_system ["show"; Fake_system.test_data "selections.xml"] in
     assert_contains "Version: 1\n" out;
     assert_contains "(not cached)" out;
 
-    let out = run_0install fake_system ["show"; feed_dir +/ "selections.xml"; "-r"] in
+    let out = run_0install fake_system ["show"; Fake_system.test_data "selections.xml"; "-r"] in
     assert_str_equal "http://example.com:8000/Hello.xml\n" out
   );
 
@@ -676,25 +679,25 @@ let suite = "0install">::: [
     assert_contains "Usage:" out;
     assert_contains "--xml" out;
 
-    let out = run_0install fake_system ["select"; feed_dir +/ "Local.xml"] in
+    let out = run_0install fake_system ["select"; Fake_system.test_data "Local.xml"] in
     assert_contains "Version: 0.1" out;
 
-    let out = run_0install fake_system ["select"; feed_dir +/ "Local.xml"; "--command="] in
+    let out = run_0install fake_system ["select"; Fake_system.test_data "Local.xml"; "--command="] in
     assert_contains "Version: 0.1" out;
 
-    let local_uri = U.realpath config.system (feed_dir +/ "Local.xml") in
-    let out = run_0install fake_system ["select"; feed_dir +/ "Local.xml"] in
+    let local_uri = U.realpath config.system (Fake_system.test_data "Local.xml") in
+    let out = run_0install fake_system ["select"; Fake_system.test_data "Local.xml"] in
     assert_contains "Version: 0.1" out;
 
-    let out = run_0install fake_system ["select"; feed_dir +/ "Local.xml"; "--xml"] in
+    let out = run_0install fake_system ["select"; Fake_system.test_data "Local.xml"; "--xml"] in
     let sels = selections_of_string ~path:local_uri out in
     let sel = Selections.(get_selected_ex {iface = local_uri; source = false}) sels in
     assert_str_equal "0.1" (Element.version sel);
 
-    let out = run_0install fake_system ["select"; feed_dir +/ "runnable/RunExec.xml"] in
+    let out = run_0install fake_system ["select"; Fake_system.test_data "runnable/RunExec.xml"] in
     assert_contains "Runner" out;
 
-    let local_uri = U.realpath config.system (feed_dir +/ "Hello.xml") in
+    let local_uri = U.realpath config.system (Fake_system.test_data "Hello.xml") in
     fake_system#putenv "DISPLAY" ":foo";
     let out = run_0install fake_system ["select"; "--xml"; local_uri] in
     let sels = selections_of_string ~path:local_uri out in
@@ -763,8 +766,8 @@ let suite = "0install">::: [
     assert_contains "FEED" out;
 
     let gpg = Support.Gpg.make config.system in
-    Support.Gpg.import_key gpg (U.read_file config.system (feed_dir +/ "6FCF121BE2390E0B.gpg")) >>= fun () ->
-    let out = run_0install fake_system ~include_stderr:true ["import"; feed_dir +/ "Hello.xml"] ~stdin:"Y\n" in
+    Support.Gpg.import_key gpg (U.read_file config.system (Fake_system.test_data "6FCF121BE2390E0B.gpg")) >>= fun () ->
+    let out = run_0install fake_system ~include_stderr:true ["import"; Fake_system.test_data "Hello.xml"] ~stdin:"Y\n" in
     assert_contains "Do you want to trust this key to sign feeds from 'example.com:8000'?" out;
     Fake_system.fake_log#assert_contains "Trusting DE937DD411906ACF7C263B396FCF121BE2390E0B for example.com:8000";
     Lwt.return ()
@@ -784,8 +787,8 @@ let suite = "0install">::: [
     assert_str_equal "" out;
 
     let gpg = Support.Gpg.make config.system in
-    Support.Gpg.import_key gpg (U.read_file config.system (feed_dir +/ "6FCF121BE2390E0B.gpg")) >>= fun () ->
-    ignore @@ run_0install fake_system ~include_stderr:true ["import"; feed_dir +/ "Hello.xml"] ~stdin:"Y\n";
+    Support.Gpg.import_key gpg (U.read_file config.system (Fake_system.test_data "6FCF121BE2390E0B.gpg")) >>= fun () ->
+    ignore @@ run_0install fake_system ~include_stderr:true ["import"; Fake_system.test_data "Hello.xml"] ~stdin:"Y\n";
 
     let out = run_0install fake_system ["list"] in
     assert_str_equal "http://example.com:8000/Hello.xml\n" out;
@@ -801,7 +804,7 @@ let suite = "0install">::: [
 
   "help">:: Fake_system.with_fake_config (fun (_config, fake_system) ->
     let out = run_0install ~exit:1 fake_system [] in
-    assert (U.starts_with out "Usage:")
+    assert (XString.starts_with out "Usage:")
   );
 
   "list2">:: Fake_system.with_fake_config ~portable_base:false (fun (config, fake_system) ->
@@ -823,12 +826,12 @@ let suite = "0install">::: [
     assert_str_equal "" out;
 
     let out = run_0install fake_system ~exit:1 ["list"; "one"; "two"] in
-    assert (U.starts_with out "Usage:");
+    assert (XString.starts_with out "Usage:");
   );
 
   "version">:: Fake_system.with_fake_config (fun (_config, fake_system) ->
     let out = run_0install fake_system ~binary:"0launch" ["--version"] in
-    assert (U.starts_with out "0launch (zero-install)");
+    assert (XString.starts_with out "0launch (zero-install)");
   );
 
   "invalid">:: Fake_system.with_fake_config (fun (_config, fake_system) ->
@@ -839,7 +842,7 @@ let suite = "0install">::: [
 
   "run">:: Fake_system.with_fake_config (fun (_config, fake_system) ->
     Fake_system.assert_raises_safe ".*test-echo' does not exist" (lazy (
-      run_0install fake_system ~binary:"0launch" [feed_dir +/ "Local.xml"] |> ignore
+      run_0install fake_system ~binary:"0launch" [Fake_system.test_data "Local.xml"] |> ignore
     ));
   );
 
@@ -876,23 +879,23 @@ let suite = "0install">::: [
 
   "need-download">:: Fake_system.with_fake_config (fun (_config, fake_system) ->
     fake_system#putenv "DISPLAY" ":foo";
-    let out = run_0install fake_system ["download"; "--dry-run"; feed_dir +/ "Foo.xml"] in
+    let out = run_0install fake_system ["download"; "--dry-run"; Fake_system.test_data "Foo.xml"] in
     assert_str_equal "" out;
   );
 
   "hello">:: Fake_system.with_fake_config (fun (_config, fake_system) ->
-    let out = run_0install fake_system ~binary:"0launch" ["--dry-run"; feed_dir +/ "Foo.xml"] in
+    let out = run_0install fake_system ~binary:"0launch" ["--dry-run"; Fake_system.test_data "Foo.xml"] in
     assert_contains "[dry-run] would execute: " out;
 
-    try run_0install fake_system ~binary:"0launch" ~exit:127 [feed_dir +/ "Foo.xml"] |> ignore
+    try run_0install fake_system ~binary:"0launch" ~exit:127 [Fake_system.test_data "Foo.xml"] |> ignore
     with Fake_system.Would_exec (false, _env, [path]) ->
       assert_contains "tests" path;
   );
 
   "ranges">:: Fake_system.with_fake_config (fun (_config, fake_system) ->
-    let out = run_0install fake_system ["select"; "--before=1"; "--not-before=0.2"; feed_dir +/ "Foo.xml"] in
-    if on_windows then assert_contains "tests\\rpm" out
-    else assert_contains "tests/rpm" out;
+    let out = run_0install fake_system ["select"; "--before=1"; "--not-before=0.2"; Fake_system.test_data "Foo.xml"] in
+    if on_windows then assert_contains "data\\rpm" out
+    else assert_contains "data/rpm" out;
   );
 
   "logging">:: Fake_system.with_fake_config (fun (_config, fake_system) ->
@@ -920,14 +923,14 @@ let suite = "0install">::: [
   );
 
   "select3">:: Fake_system.with_fake_config (fun (_config, fake_system) ->
-    let command_feed = feed_dir +/ "Command.xml" in
+    let command_feed = Fake_system.test_data "Command.xml" in
     let out = run_0install fake_system ["select"; command_feed] in
     assert_contains "Local.xml" out
   );
 
   "help3">:: Fake_system.with_fake_config (fun (_config, fake_system) ->
     let out = run_0install fake_system ~exit:1 [] in
-    assert (U.starts_with out "Usage:");
+    assert (XString.starts_with out "Usage:");
     assert_contains "add-feed" out;
     assert_contains "--version" out;
 
@@ -944,17 +947,17 @@ let suite = "0install">::: [
 
   "run2">:: Fake_system.with_fake_config (fun (_config, fake_system) ->
     let out = run_0install fake_system ~exit:1 ["run"] in
-    assert (U.starts_with out "Usage:");
+    assert (XString.starts_with out "Usage:");
     assert_contains "URI" out;
 
-    let out = run_0install fake_system ["run"; "--dry-run"; feed_dir +/ "runnable/Runnable.xml"; "--help"] in
+    let out = run_0install fake_system ["run"; "--dry-run"; Fake_system.test_data "runnable/Runnable.xml"; "--help"] in
     assert_contains "arg-for-runner" out;
     assert_contains "--help" out;
   );
 
   "update-alias">:: Fake_system.with_fake_config (fun (_config, fake_system) ->
     skip_if on_windows "Aliases don't work on Windows";
-    let local_feed = feed_dir +/ "Local.xml" in
+    let local_feed = Fake_system.test_data "Local.xml" in
     let bindir = fake_system#tmpdir +/ "bin" in
     fake_system#mkdir bindir 0o700;
     let launcher_script = bindir +/ "my-test-alias" in
@@ -968,24 +971,24 @@ let suite = "0install">::: [
   "man">:: Fake_system.with_fake_config (fun (config, fake_system) ->
     skip_if on_windows "No man. No aliases";
     let out = run_0install fake_system ["man"; "--help"] in
-    assert (U.starts_with out "Usage:");
+    assert (XString.starts_with out "Usage:");
 
     (* Wrong number of args: pass-through *)
     check_man fake_system ["git"; "config"] ["man"; "git"; "config"];
     check_man fake_system [] ["man"];
 
-    let local_feed = feed_dir +/ "Local.xml" in
+    let local_feed = Fake_system.test_data "Local.xml" in
     let bindir = fake_system#tmpdir +/ "bin" in
     fake_system#mkdir bindir 0o700;
     let launcher_script = bindir +/ "my-test-alias" in
     write_script fake_system launcher_script local_feed;
-    check_man fake_system ["my-test-alias"] ["man"; "tests/test-echo.1"];
+    check_man fake_system ["my-test-alias"] ["man"; "tests/data/test-echo.1"];
 
     check_man fake_system ["__i_dont_exist"] ["man"; "__i_dont_exist"];
     check_man fake_system ["ls"] ["man"; "ls"];
 
     (* No man-page *)
-    let binary_feed = feed_dir +/ "Command.xml" |> U.realpath config.system in
+    let binary_feed = Fake_system.test_data "Command.xml" |> U.realpath config.system in
     let launcher_script = bindir +/ "my-binary-alias" in
     write_script fake_system launcher_script binary_feed;
 
@@ -998,7 +1001,7 @@ let suite = "0install">::: [
     Unix.mkdir (tmpdir +/ "bin") 0o700;
     fake_system#putenv "PATH" ((tmpdir +/ "bin") ^ path_sep ^ U.getenv_ex fake_system "PATH");
 
-    let local_feed = feed_dir +/ "Local.xml" in
+    let local_feed = Fake_system.test_data "Local.xml" in
     let out = run_0install fake_system ~binary:"0alias" ["local-app"; local_feed] in
     assert_str_equal "(\"0alias\" is deprecated; using \"0install add\" instead)\n" out;
   );
