@@ -4,6 +4,7 @@
 
 (** Explaining why a solve failed or gave an unexpected answer. *)
 
+open Support
 open Support.Common
 
 module U = Support.Utils
@@ -11,9 +12,9 @@ module U = Support.Utils
 module Make (Model : Sigs.SOLVER_RESULT) = struct
   module RoleMap = Model.RoleMap
 
-  let format_role f role = Format.pp_print_string f (Model.Role.to_string role)
+  let format_role = Model.Role.pp
 
-  let spf = Support.Logging.asprintf
+  let spf = Format.asprintf
 
   (* Why a particular implementation was rejected. This could be because the model rejected it,
      or because it conflicts with something else in the example (partial) solution. *)
@@ -161,9 +162,9 @@ module Make (Model : Sigs.SOLVER_RESULT) = struct
     end
 
   let find_component_ex role report =
-    match RoleMap.find role report with
+    match RoleMap.find_opt role report with
     | Some c -> c
-    | None -> raise_safe "Can't find component %a!" format_role role
+    | None -> Safe_exn.failf "Can't find component %a!" format_role role
 
   (* Did any dependency of [impl] prevent it being selected?
      This can only happen if a component conflicts with something more important
@@ -176,7 +177,7 @@ module Make (Model : Sigs.SOLVER_RESULT) = struct
   let get_dependency_problem role report impl =
     let check_dep dep =
       let dep_info = Model.dep_info dep in
-      match RoleMap.find dep_info.Model.dep_role report with
+      match RoleMap.find_opt dep_info.Model.dep_role report with
       | None -> None      (* Not in the selections => can't be part of a conflict *)
       | Some required_component ->
           match required_component#impl with
@@ -199,7 +200,7 @@ module Make (Model : Sigs.SOLVER_RESULT) = struct
       of the required interface were rejected. *)
   let examine_dep requiring_role requiring_impl report dep =
     let {Model.dep_role = other_role; dep_importance = _; dep_required_commands} = Model.dep_info dep in
-    match RoleMap.find other_role report with
+    match RoleMap.find_opt other_role report with
     | None -> ()
     | Some required_component ->
         let dep_restrictions = Model.restrictions dep in
@@ -227,7 +228,7 @@ module Make (Model : Sigs.SOLVER_RESULT) = struct
       | Some replacement when RoleMap.mem replacement report -> (
           component#note (ReplacedByConflict replacement);
           component#reject_all (`ConflictsRole replacement);
-          match RoleMap.find replacement report with
+          match RoleMap.find_opt replacement report with
           | Some replacement_component ->
               replacement_component#note (ReplacesConflict role);
               replacement_component#reject_all (`ConflictsRole role)
