@@ -5,6 +5,7 @@
 (** Interacting with distribution package managers. *)
 
 open General
+open Support
 open Support.Common
 module U = Support.Utils
 
@@ -96,9 +97,9 @@ let try_cleanup_distro_version_warn version package_name =
 
 let iter_dir system fn path =
   match system#readdir path with
-  | Problem ex when U.is_dir system path -> raise ex
-  | Problem ex -> log_debug ~ex "Failed to read directory '%s'" path
-  | Success items -> items |> Array.iter fn
+  | Error ex when U.is_dir system path -> raise ex
+  | Error ex -> log_debug ~ex "Failed to read directory '%s'" path
+  | Ok items -> items |> Array.iter fn
 
 (** Lookup [elem]'s package in the cache. Generate the ID(s) for the cached implementations and check that one of them
     matches the [id] attribute on [elem].
@@ -107,7 +108,7 @@ let check_cache id_prefix elem cache =
   let package = Element.package elem in
   let sel_id = Element.id elem in
   let matches (installed_version, machine) =
-    let installed_id = Support.Logging.asprintf "%s:%s:%a:%s" id_prefix package Version.fmt installed_version (Arch.format_machine_or_star machine) in
+    let installed_id = Format.asprintf "%s:%s:%a:%s" id_prefix package Version.pp installed_version (Arch.format_machine_or_star machine) in
     (* log_warning "Want %s %s, have %s" package sel_id installed_id; *)
     sel_id = installed_id in
   List.exists matches (fst (Distro_cache.get cache package))
@@ -143,29 +144,42 @@ module Debian = struct
       size : Int64.t option;
     }
 
-    type t = (string, [`Available of entry | `Unavailable]) Hashtbl.t
+    type t = (string, [`Available of entry | `Unavailable] Lwt.t) Hashtbl.t
 
     let make () = Hashtbl.create 10
 
+    let parse_cache_line line =
+      match XString.(split_pair re_space) (String.trim line) with
+      | None -> `Unknown
+      | Some (k, v) ->
+        let v = String.trim v in
+        match k with
+        | "Version:" -> `Version v
+        | "Architecture:" -> `Architecture v
+        | "Size:" -> `Size (Int64.of_string v)
+        | _ -> `Unknown
+
+    (* Avoid running too many apt-cache processes at the same time. *)
+    let mux = Lwt_mutex.create ()
+
     let run system package =
       Lwt.catch
         (fun () ->
            let stderr = if Support.Logging.will_log Support.Logging.Debug then None else Some `Dev_null in
-           Lwt_process.pread ?stderr (U.make_command system ["apt-cache"; "--no-all-versions"; "show"; "--"; package]) >>= fun out ->
+           Lwt_mutex.with_lock mux (fun () ->
+               Lwt_process.pread ?stderr (U.make_command system ["apt-cache"; "--no-all-versions"; "show"; "--"; package])
+             ) >>= fun out ->
            let machine = ref None in
            let version = ref None in
            let size = ref None in
            let stream = U.stream_of_lines out in
            begin try
                while true do
-                 let line = Stream.next stream |> String.trim in
-                 if U.starts_with line "Version: " then (
-                   version := try_cleanup_distro_version_warn (U.string_tail line 9 |> String.trim) package
-                 ) else if U.starts_with line "Architecture: " then (
-                   machine := Some (Support.System.canonical_machine (U.string_tail line 14 |> String.trim))
-                 ) else if U.starts_with line "Size: " then (
-                   size := Some (Int64.of_string (U.string_tail line 6 |> String.trim))
-                 )
+                 match parse_cache_line (Stream.next stream) with
+                 | `Version x -> version := try_cleanup_distro_version_warn x package
+                 | `Architecture x -> machine := Some (Support.System.canonical_machine x)
+                 | `Size x -> size := Some x
+                 | `Unknown -> ()
                done
              with Stream.Failure -> ()
            end;
@@ -186,10 +200,13 @@ module Debian = struct
       Lwt.catch
         (fun () ->
            package_names |> Lwt_list.iter_s (fun package ->
-               run system package >>= fun result ->
                (* (multi-arch support? can there be multiple candidates?) *)
-               Hashtbl.replace t package result;
-               Lwt.return ()
+               match Hashtbl.find_opt t package with
+               | Some result when Lwt.state result = Lwt.Sleep -> result >|= ignore  (* Already checking *)
+               | _ ->
+                 let result = run system package in
+                 Hashtbl.replace t package result;
+                 result >|= ignore
              )
            >|= fun () -> `Ok
         )
@@ -199,9 +216,12 @@ module Debian = struct
         )
 
     let cached_result t package_name =
-      match Hashtbl.find t package_name with
-      | exception Not_found -> `Not_checked
-      | x -> (x :> [`Available of entry | `Unavailable | `Not_checked])
+      match Hashtbl.find_opt t package_name with
+      | None -> `Not_checked
+      | Some x ->
+        match Lwt.state x with
+        | Lwt.Return x -> (x :> [`Available of entry | `Unavailable | `Not_checked])
+        | Lwt.Sleep | Lwt.Fail _ -> `Not_checked
 
     let is_available t package_name =
       match cached_result t package_name with
@@ -236,12 +256,12 @@ module Debian = struct
           try
             while true do
               let line = input_line ch in
-              match Str.bounded_split_delim U.re_tab line 3 with
+              match Str.bounded_split_delim XString.re_tab line 3 with
               | [] -> ()
               | [version; debarch; status] ->
-                if U.ends_with status " installed" then (
+                if XString.ends_with status " installed" then (
                   let debarch =
-                    try U.string_tail debarch (String.rindex debarch '-' + 1)
+                    try XString.tail debarch (String.rindex debarch '-' + 1)
                     with Not_found -> debarch in
                   match try_cleanup_distro_version_warn version package_name with
                   | None -> ()
@@ -366,9 +386,9 @@ module Debian = struct
 
       method! private get_correct_main impl run_command =
         let id = Impl.get_attr_ex Constants.FeedAttr.id impl in
-        if U.starts_with id "package:deb:openjdk-6-jre:" then
+        if XString.starts_with id "package:deb:openjdk-6-jre:" then
           fixup_java_main impl "6-openjdk"
-        else if U.starts_with id "package:deb:openjdk-7-jre:" then
+        else if XString.starts_with id "package:deb:openjdk-7-jre:" then
           fixup_java_main impl "7-openjdk"
         else
           super#get_correct_main impl run_command
@@ -403,7 +423,7 @@ module RPM = struct
         try
           while true do
             let line = input_line from_rpm in
-            match Str.bounded_split_delim U.re_tab line 3 with
+            match Str.bounded_split_delim XString.re_tab line 3 with
             | ["gpg-pubkey"; _; _] -> ()
             | [package; version; rpmarch] ->
                 let zi_arch = Support.System.canonical_machine (String.trim rpmarch) |> Arch.parse_machine in
@@ -440,7 +460,7 @@ module RPM = struct
       method! private get_correct_main impl run_command =
         (* OpenSUSE uses _, Fedora uses . *)
         let id = Impl.get_attr_ex Constants.FeedAttr.id impl in
-        let starts x = U.starts_with id x in
+        let starts x = XString.starts_with id x in
         if starts "package:rpm:java-1.6.0-openjdk:" || starts "package:rpm:java-1_6_0-openjdk:" then
           fixup_java_main impl "1.6.0-openjdk"
         else if starts "package:rpm:java-1.7.0-openjdk:" || starts "package:rpm:java-1_7_0-openjdk:" then
@@ -475,7 +495,7 @@ module ArchLinux = struct
         let build_dash = String.rindex entry '-' in
         let version_dash = String.rindex_from entry (build_dash - 1) '-' in
         Some (String.sub entry 0 version_dash,
-              U.string_tail entry (version_dash + 1))
+              XString.tail entry (version_dash + 1))
       with Not_found -> None in
 
     let get_arch desc_path =
@@ -491,21 +511,21 @@ module ArchLinux = struct
       );
       !arch in
 
-    let entries = ref (-1.0, StringMap.empty) in
+    let entries = ref (-1.0, XString.Map.empty) in
     let get_entries () =
       let (last_read, items) = !entries in
       match config.system#stat packages_dir with
       | Some info when info.Unix.st_mtime > last_read -> (
           match config.system#readdir packages_dir with
-          | Success items ->
+          | Ok items ->
               let add map entry =
                 match parse_dirname entry with
-                | Some (name, version) -> StringMap.add name version map
+                | Some (name, version) -> XString.Map.add name version map
                 | None -> map in
-              let new_items = Array.fold_left add StringMap.empty items in
+              let new_items = Array.fold_left add XString.Map.empty items in
               entries := (info.Unix.st_mtime, new_items);
               new_items
-          | Problem ex ->
+          | Error ex ->
               log_warning ~ex "Can't read packages dir '%s'!" packages_dir;
               items
       )
@@ -526,7 +546,7 @@ module ArchLinux = struct
         let package_name = Query.package query in
         log_debug "Looking up distribution packages for %s" package_name;
         let items = get_entries () in
-        match StringMap.find package_name items with
+        match XString.Map.find_opt package_name items with
         | None -> ()
         | Some version ->
             let entry = package_name ^ "-" ^ version in
@@ -553,7 +573,7 @@ module Mac = struct
       let first_line = [main; "--version"] |> U.check_output config.system input_line in
       try
         let i = String.rindex first_line ' ' in
-        U.string_tail first_line (i + 1)
+        XString.tail first_line (i + 1)
       with Not_found -> first_line in
 
     let java_home version arch =
@@ -581,7 +601,7 @@ module Mac = struct
         | "gnupg2" -> self#find_program "/usr/local/bin/gpg2" query
         | "make" -> self#find_program "/usr/bin/make" query
         | package_name ->
-            if StringMap.is_empty (Query.results query) then
+            if XString.Map.is_empty (Query.results query) then
               Query.problem query "%s: Unknown Darwin/OS X package" package_name
             (* else we have MacPorts results *)
 
@@ -634,10 +654,10 @@ module Mac = struct
           while true do
             let line = input_line ch in
             log_debug "Got: '%s'" line;
-            if U.starts_with line " " then (
+            if XString.starts_with line " " then (
               let line = String.trim line in
-              match Str.bounded_split_delim U.re_space line 3 with
-              | [package; version; extra] when U.starts_with extra "(active)" ->
+              match Str.bounded_split_delim XString.re_space line 3 with
+              | [package; version; extra] when XString.starts_with extra "(active)" ->
                   log_debug "Found package='%s' version='%s' extra='%s'" package version extra;
                   if Str.string_match re_version version 0 then (
                     let version = Str.matched_group 1 version in
@@ -646,7 +666,7 @@ module Mac = struct
                         (* let platform = Str.matched_group 1 extra in *)
                         (* let major = Str.matched_group 2 extra in *)
                         let archs = Str.matched_group 3 extra in
-                        Str.split U.re_space archs |> List.iter (fun arch ->
+                        Str.split XString.re_space archs |> List.iter (fun arch ->
                           let zi_arch = Support.System.canonical_machine arch in
                           add_entry package (version, Arch.parse_machine zi_arch)
                         )
@@ -657,7 +677,7 @@ module Mac = struct
                   ) else log_debug "Failed to match version '%s'" version
               | [_package; _version; _extra] -> ()
               | [_package; _version] -> ()
-              | _ -> raise_safe "Invalid port output: '%s'" line
+              | _ -> Safe_exn.failf "Invalid port output: '%s'" line
             )
           done
         with End_of_file -> ()
@@ -695,7 +715,7 @@ end
 
 module Win = struct
   let windows_distribution config =
-    let api = !Support.Windows_api.windowsAPI |? lazy (raise_safe "Failed to load Windows support module!") in
+    let api = !Support.Windows_api.windowsAPI |? lazy (Safe_exn.failf "Failed to load Windows support module!") in
 
     let read_hklm_reg reader =
       let open Support.Windows_api in
@@ -817,7 +837,7 @@ module Win = struct
             match input_line from_cyg with
             | "Cygwin Package Information" | "" -> ()
             | line ->
-                match U.split_pair re_whitespace line with
+                match XString.split_pair_safe re_whitespace line with
                 | ("Package", "Version") -> ()
                 | (package, version) ->
                     try_cleanup_distro_version_warn version package |> if_some (fun clean_version ->
@@ -910,29 +930,29 @@ module Gentoo = struct
         let re_version_start = Str.regexp "-[0-9]" in
 
         let package_name = Query.package query in
-        match Str.bounded_split_delim U.re_slash package_name 2 with
+        match Str.bounded_split_delim XString.re_slash package_name 2 with
         | [category; leafname] ->
             let category_dir = pkgdir +/ category in
             let match_prefix = leafname ^ "-" in
 
             category_dir |> iter_dir config.system (fun filename ->
-              if U.starts_with filename match_prefix && is_digit (filename.[String.length match_prefix]) then (
+              if XString.starts_with filename match_prefix && is_digit (filename.[String.length match_prefix]) then (
                 let pf_path = category_dir +/ filename +/ "PF"in
-                let pf_mtime = (config.system#lstat pf_path |? lazy (raise_safe "Missing '%s' file!" pf_path)).Unix.st_mtime in
+                let pf_mtime = (config.system#lstat pf_path |? lazy (Safe_exn.failf "Missing '%s' file!" pf_path)).Unix.st_mtime in
                 let name = pf_path|> config.system#with_open_in [Open_rdonly] input_line |> String.trim in
 
                 match (try Some (Str.search_forward re_version_start name 0) with Not_found -> None) with
                 | None -> log_warning "Cannot parse version from Gentoo package named '%s'" name
                 | Some i ->
-                  try_cleanup_distro_version_warn (U.string_tail name (i + 1)) package_name |> if_some (fun version ->
+                  try_cleanup_distro_version_warn (XString.tail name (i + 1)) package_name |> if_some (fun version ->
                     let machine =
-                      if category = "app-emulation" && U.starts_with name "emul-" then (
-                        match Str.bounded_split_delim U.re_dash name 4 with
+                      if category = "app-emulation" && XString.starts_with name "emul-" then (
+                        match Str.bounded_split_delim XString.re_dash name 4 with
                         | [_; _; machine; _] -> machine
                         | _ -> "*"
                       ) else (
                         category_dir +/ filename +/ "CHOST" |> config.system#with_open_in [Open_rdonly] (fun ch ->
-                          input_line ch |> U.split_pair U.re_dash |> fst
+                          input_line ch |> XString.(split_pair_safe re_dash) |> fst
                         )
                       ) in
                     let machine = Arch.parse_machine (Support.System.canonical_machine machine) in
@@ -965,7 +985,7 @@ module Slackware = struct
         let package_name = Query.package query in
         super#get_package_impls query;
         packages_dir |> iter_dir config.system (fun entry ->
-          match Str.bounded_split_delim U.re_dash entry 4 with
+          match Str.bounded_split_delim XString.re_dash entry 4 with
           | [name; version; arch; build] when name = package_name ->
               let machine = Arch.parse_machine (Support.System.canonical_machine arch) in
               try_cleanup_distro_version_warn (version ^ "-" ^ build) package_name |> if_some (fun version ->
