@@ -4,6 +4,7 @@
 
 (** Helpers for --dry-run mode. *)
 
+open Support
 open Support.Common
 
 (** Log a message saying what we would have done. *)
@@ -14,15 +15,13 @@ let log fmt =
 
 (** Wrap a system and prevent all changes (writes). This is used with --dry-run to prevent accidents. *)
 class dryrun_system (underlying:system) =
-  let reject msg = raise_safe "Bug: '%s' called in --dry-run mode" msg in
+  let reject msg = Safe_exn.failf "Bug: '%s' called in --dry-run mode" msg in
   object (_ : #system)
-    val mutable fake_dirs = StringMap.empty
+    val mutable fake_dirs = XString.Map.empty
 
     (* Read-only operations: pass though *)
     method argv = underlying#argv
     method isatty = underlying#isatty
-    method print_string = underlying#print_string
-    method std_formatter = underlying#std_formatter
     method time = underlying#time
     method with_open_in = underlying#with_open_in
     method readdir = underlying#readdir
@@ -44,8 +43,8 @@ class dryrun_system (underlying:system) =
         let dir = Filename.dirname path in
         let base = Filename.basename path in
 
-        match StringMap.find dir fake_dirs with
-        | Some items -> StringSet.mem base items
+        match XString.Map.find_opt dir fake_dirs with
+        | Some items -> XString.Set.mem base items
         | None -> false
       )
 
@@ -63,9 +62,9 @@ class dryrun_system (underlying:system) =
       let dir = Filename.dirname path in
       let base = Filename.basename path in
 
-      let dir_entries = default StringSet.empty @@ StringMap.find dir fake_dirs in
+      let dir_entries = default XString.Set.empty @@ XString.Map.find_opt dir fake_dirs in
 
-      fake_dirs <- StringMap.add dir (StringSet.add base dir_entries) fake_dirs
+      fake_dirs <- XString.Map.add dir (XString.Set.add base dir_entries) fake_dirs
 
     (* Interesting operations: log and skip *)
     method hardlink orig copy = log "ln %s %s" orig copy
